/*
 * Project: ClusterControl
 * Version: 2.0
 * Author : Gernot Stocker, gernot.stocker@tugraz.at (c) 2006
 * Date   :
 * Description: command loop which can be run on every node
 */

#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>
#include "defines.h"
#include "utils.h"
#include <stdio.h>

/*
    CPU0UserTicks = $2
    CPU0SystemTicks = $3
    CPU0NiceTicks = $4
    CPU0IdleTicks = $5 
*/

extern float cpu1_user_tick;
extern float cpu1_nice_tick;
extern float cpu1_idle_tick;
extern float cpu1_system_tick;
extern float cpu1_load;

#ifdef DUAL_CPU
extern float cpu2_user_tick;
extern float cpu2_system_tick;
extern float cpu2_nice_tick;
extern float cpu2_idle_tick;
extern float cpu2_load;
#endif

void calculateCPUString(char *status_line);
void getDiskUsage(char *status_line);

int commandloop(char *hostname, char *input_buffer, int connection_counter, fd_set *open_sockets)
{
  int    status=0;
  char   *status_line=getString();

  if (strstr(input_buffer,"mem")!=0)
  {
    writeLogMessage("MEM");
    write(connection_counter, "mem:\n", strlen("mem:\n"));

    readLineFromFile("/proc/meminfo",status_line,1);
    write(connection_counter, status_line, strlen(status_line));
    write(connection_counter, "::", 2);

    readLineFromFile("/proc/meminfo",status_line,2);
    write(connection_counter, status_line, strlen(status_line));
    write(connection_counter, "::", 2);

    readLineFromFile("/proc/meminfo",status_line,3);
    write(connection_counter, status_line, strlen(status_line));
    write(connection_counter, "::", 2);

    readLineFromFile("/proc/meminfo",status_line,4);
    write(connection_counter, status_line, strlen(status_line));
    status=1;
  }
	 
  if (strstr(input_buffer,"uptime")!=0)
  {
    writeLogMessage("UPTIME");
    write(connection_counter, "uptime:\n", strlen("uptime:\n"));
    readLineFromFile("/proc/uptime",status_line,2);
    write(connection_counter, status_line, strlen(status_line));
    status=1;
  }

  if (strstr(input_buffer,"load")!=0)
  { 
    writeLogMessage("LOAD");
    write(connection_counter, "load:\n", strlen("load:\n"));    
    readLineFromFile("/proc/loadavg",status_line,1);
    write(connection_counter, status_line, strlen(status_line));
    status=1;
  }

  if (strstr(input_buffer,"cpu")!=0)
  { 
    writeLogMessage("CPU");
    calculateCPUString(status_line);
    write(connection_counter, status_line, strlen(status_line));
    status=1;
  }

  if (strstr(input_buffer,"disk")!=0)
  {
    writeLogMessage("DISK");
    getDiskUsage(status_line);
    write(connection_counter, status_line, strlen(status_line));
    status=1;
  }

  if (strstr(input_buffer,"help")!=0)
  {
    writeLogMessage("HELP");
    write(connection_counter, "help:\n", strlen("help:\n"));
    snprintf(status_line, 1023, "ValidCommands: mem, cpu, uptime, load, disk, exit, quit, help");
    write(connection_counter, status_line, strlen(status_line));
    status=1;
  }

  if (
       (strstr(input_buffer,"exit")!=0)||
       (strstr(input_buffer,"quit")!=0)||
       (strstr(input_buffer,"bye")!=0)
     )
  {
    close(connection_counter);
    FD_CLR(connection_counter, open_sockets);   
    free(status_line);
    return(1);
  }
  else
  {
    if (status==0)
    {
      write(connection_counter, "NO invalid command\n", strlen("NO invalid command\n"));
    } 
    else
    {
      write(connection_counter, "\nOK\n", strlen("\nOK\n"));	  
    }
    write(connection_counter, hostname, strlen(hostname));	  
    write(connection_counter, ":> ", strlen(":> "));
  }
  free(status_line);
  return(0);
}


void initCPUValues()
{
  char *status_line=getString();
  char cpu[20];
  char cpu1[20];
  char cpu2[20];
  char cpu3[20];
  char cpu4[20];
  
#ifdef DUAL_CPU
  readLineFromFile("/proc/stat",status_line,2);
  sscanf(status_line,"%s  %s %s %s %s", cpu, cpu1, cpu2, cpu3, cpu4);
  cpu1_user_tick=atof(cpu1);
  cpu1_system_tick=atof(cpu2);
  cpu1_nice_tick=atof(cpu3);
  cpu1_idle_tick=atof(cpu4);

  readLineFromFile("/proc/stat",status_line,3);
  sscanf(status_line,"%s  %s %s %s %s", cpu, cpu1, cpu2, cpu3, cpu4);
  cpu2_user_tick=atof(cpu1);
  cpu2_system_tick=atof(cpu2);
  cpu2_nice_tick=atof(cpu3);
  cpu2_idle_tick=atof(cpu4);

#else
  readLineFromFile("/proc/stat",status_line,1);
  sscanf(status_line,"%s  %s %s %s %s", cpu, cpu1, cpu2, cpu3, cpu4);
  cpu1_user_tick=atof(cpu1);
  cpu1_system_tick=atof(cpu2);
  cpu1_nice_tick=atof(cpu3);
  cpu1_idle_tick=atof(cpu4);
#endif
  free(status_line);
}

void setNewCPUValues()
{
  char *status_line=getString();
  char cpu[20];
  char cpu1[20];
  char cpu2[20];
  char cpu3[20];
  char cpu4[20];
  float cpu1_user_tick_new=0;
  float cpu1_system_tick_new=0;
  float cpu1_nice_tick_new=0;
  float cpu1_idle_tick_new=0;
  float cpu1_total_tick=0;
#ifdef DUAL_CPU
  float cpu2_user_tick_new=0;
  float cpu2_system_tick_new=0;
  float cpu2_nice_tick_new=0;
  float cpu2_idle_tick_new=0;
  float cpu2_total_tick=0;
  /*
     GetInfo CPU1
  */
  readLineFromFile("/proc/stat",status_line,2);
  sscanf(status_line,"%s  %s %s %s %s", cpu, cpu1, cpu2, cpu3, cpu4);
  cpu1_user_tick_new=atof(cpu1);
  cpu1_system_tick_new=atof(cpu2);
  cpu1_nice_tick_new=atof(cpu3);
  cpu1_idle_tick_new=atof(cpu4);
  /*
     Calculation CPU1
  */    
  cpu1_total_tick=((cpu1_user_tick_new   - cpu1_user_tick)+(cpu1_system_tick_new - cpu1_system_tick)+(cpu1_nice_tick_new   - cpu1_nice_tick)+(cpu1_idle_tick_new   - cpu1_idle_tick));
  if (cpu1_total_tick!=0)
  {
    cpu1_load=(((cpu1_user_tick_new   - cpu1_user_tick)+(cpu1_system_tick_new -
    cpu1_system_tick)+(cpu1_nice_tick_new   - cpu1_nice_tick)) / (cpu1_total_tick / 100));
  }
  else
  {
    cpu1_load=0;
  }
  /*
     Assignment CPU1
  */
  cpu1_user_tick=cpu1_user_tick_new;
  cpu1_system_tick=cpu1_system_tick_new;
  cpu1_nice_tick=cpu1_nice_tick_new;
  cpu1_idle_tick=cpu1_idle_tick_new;
  /*
     GetInfo CPU2
  */
  readLineFromFile("/proc/stat",status_line,3);
  sscanf(status_line,"%s  %s %s %s %s", cpu, cpu1, cpu2, cpu3, cpu4);
  cpu2_user_tick_new=atof(cpu1);
  cpu2_system_tick_new=atof(cpu2);
  cpu2_nice_tick_new=atof(cpu3);
  cpu2_idle_tick_new=atof(cpu4);
  /*
     Calculation CPU2
  */    
  cpu2_total_tick=((cpu2_user_tick_new   - cpu2_user_tick)+(cpu2_system_tick_new - cpu2_system_tick)+(cpu2_nice_tick_new   - cpu2_nice_tick)+(cpu2_idle_tick_new   - cpu2_idle_tick));  
  if (cpu2_total_tick!=0)
  {
    cpu2_load=(((cpu2_user_tick_new   - cpu2_user_tick)+(cpu2_system_tick_new -
    cpu2_system_tick)+(cpu2_nice_tick_new   - cpu2_nice_tick)) / (cpu2_total_tick / 100));		 
  }
  else
  {
    cpu2_load=0;
  }
  /*
     Assignment CPU2
  */
  cpu2_user_tick=cpu2_user_tick_new;
  cpu2_system_tick=cpu2_system_tick_new;
  cpu2_nice_tick=cpu2_nice_tick_new;
  cpu2_idle_tick=cpu2_idle_tick_new;

#else
  /*
     GetInfo CPU1
  */
  readLineFromFile("/proc/stat",status_line,1);
  sscanf(status_line,"%s  %s %s %s %s", cpu, cpu1, cpu2, cpu3, cpu4);
  cpu1_user_tick_new=atof(cpu1);
  cpu1_system_tick_new=atof(cpu2);
  cpu1_nice_tick_new=atof(cpu3);
  cpu1_idle_tick_new=atof(cpu4);
  /*
     Calculation
  */    
  cpu1_total_tick=((cpu1_user_tick_new   - cpu1_user_tick)+(cpu1_system_tick_new - cpu1_system_tick)+(cpu1_nice_tick_new   - cpu1_nice_tick)+(cpu1_idle_tick_new   - cpu1_idle_tick));
  if (cpu1_total_tick!=0)
  {
    cpu1_load=(((cpu1_user_tick_new   - cpu1_user_tick)+(cpu1_system_tick_new -
    cpu1_system_tick)+(cpu1_nice_tick_new   - cpu1_nice_tick)) / (cpu1_total_tick / 100));
  }
  else
  {
    cpu1_load=0;
  }
  /*
     Assignment
  */
  cpu1_user_tick=cpu1_user_tick_new;
  cpu1_system_tick=cpu1_system_tick_new;
  cpu1_nice_tick=cpu1_nice_tick_new;
  cpu1_idle_tick=cpu1_idle_tick_new;
#endif
  free(status_line);
}



void calculateCPUString(char *status_line)
{
#ifdef DUAL_CPU
  sprintf(status_line, "cpu:\n %10.0f  %10.0f", cpu1_load, cpu2_load); 
#else
  sprintf(status_line, "cpu:\n %10.0f", cpu1_load); 
#endif
  writeLogMessage(status_line);
}


void getDiskUsage(char *status_line)
{
  FILE *fp;
  char output[INPUT_BUFFER_LENGTH];
  fp = popen(DF_PROGRAM, "r");
  sprintf(status_line,"disk:\n");
  while(!feof(fp))
  {
   fgets(output,  INPUT_BUFFER_LENGTH, fp);
   if (strstr(output,"/dev/"))
   {
     writeLogMessage(output);
     strncat(status_line, output, INPUT_BUFFER_LENGTH);
   } 
  }
  pclose(fp);
}
