/*
 * Project: ClusterControl
 * Version: 2.0
 * Author : Gernot Stocker, gernot.stocker@tugraz.at (c) 2006
 * Date   :
 * Description: Utilities
 */
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <string.h>
#include <sys/time.h>

#include "defines.h"
#include "network.h"
#include "commandloop.h"
#include "utils.h"



/*
   Global Daemon Signal Handler
*/
volatile sig_atomic_t run_daemon = 1; 
int    server_socket;
fd_set open_sockets;
char   *hostname;
float cpu1_user_tick=0;
float cpu1_system_tick=0;
float cpu1_nice_tick=0;
float cpu1_idle_tick=0;
float cpu1_load=0;

#ifdef DUAL_CPU
float cpu2_user_tick=0;
float cpu2_system_tick=0;
float cpu2_nice_tick=0;
float cpu2_idle_tick=0;
float cpu2_load=0;
#endif


void handleTermSignal (int signal_value)
{
  int open_sockets_counter;
  int connection_counter; 
  signal (signal_value, handleTermSignal);
  run_daemon = 0;
  writeLogMessage("TermSignal!!!!!");

  open_sockets_counter=getdtablesize();
  for (connection_counter=0; connection_counter < open_sockets_counter; connection_counter++)
  {
    writeLogMessage("Closed");
    if (connection_counter!=server_socket) close(connection_counter);
  }
  close(server_socket);
  free(hostname);
  writeLogMessage("Termed!!!!!");
}

void initProcessSignals()
{
  if (signal (SIGTERM, handleTermSignal) == SIG_IGN) signal (SIGTERM, SIG_IGN);
  signal (SIGINT, SIG_IGN);
  signal (SIGHUP, SIG_IGN);  
}

int
main (void)
{
  int    connection_counter=0;
  struct sockaddr_in server_address;
  struct timeval  timeout;
  fd_set waiting_open_sockets;
  int    open_sockets_counter=0;
  char   input_buffer[INPUT_BUFFER_LENGTH + 1];
  int    error_value;

  if (chdir (WORKING_DIR))         
  {
    fprintf (stderr, "Could'nt change to `%s': ", WORKING_DIR);
    perror (NULL);
    exit (4);
  }

  if (DAEMON)
  {
    writeLogMessage("Daemon!!!!!");
    switch (fork ())
    {
      case -1:                    
            perror ("fork()");
            exit (5);
      case 0:                            // child process     
            close (STDIN_FILENO);
            close (STDOUT_FILENO);
            close (STDERR_FILENO);
            if (setsid () == -1)      
              {
                exit (6);
              }
            break;
      default:                           // parent process
            return 0;
    }
    
  }
  else
  {
    writeLogMessage("NO Daemon!!!!!");
  }
   
  initProcessSignals();
  writeLogMessage("INIT Process!!!!!");
  initCPUValues();
  writeLogMessage("SetValue!!!!!");
  initServerConnection(&server_address);  
  writeLogMessage("InitServer!!!!!");
  connectServerSocket(&server_address);
  writeLogMessage("ConnectServer!!!!!");
  open_sockets_counter=getdtablesize();
  writeLogMessage("GetSockets!!!!!");
  
  FD_ZERO(&open_sockets);
  FD_SET(server_socket, &open_sockets);
  FD_ZERO(&waiting_open_sockets);
  timeout.tv_sec  = 2;
  timeout.tv_usec = 0;

  while (run_daemon)
  { 
    writeLogMessage("Server Loop!!!!!");
    waiting_open_sockets=open_sockets;
    error_value=select(open_sockets_counter, &waiting_open_sockets, NULL, NULL, &timeout);

    if (error_value==0) 
    {
      timeout.tv_sec  = 1; timeout.tv_usec = 0;
      setNewCPUValues();
      writeLogMessage("TimedOut!!!!!");
    }
    
    if (FD_ISSET(server_socket, &waiting_open_sockets))
    {
      sprintf(input_buffer,"%s (%s)\n%s:> ",SERVER_HELLO , hostname, hostname);
      write( addClientSocket(&open_sockets), input_buffer, strlen(input_buffer));	  
      continue;
    }

    for (connection_counter=0; connection_counter<open_sockets_counter; connection_counter++)
    {
      if (connection_counter != server_socket && FD_ISSET(connection_counter, &waiting_open_sockets)) 
      {
	error_value = read(connection_counter, input_buffer, INPUT_BUFFER_LENGTH);
	if (error_value == 0) 
	{
          close(connection_counter);
          FD_CLR(connection_counter, &open_sockets);
	}
        else 
	{
          commandloop(hostname, input_buffer, connection_counter, &open_sockets);
	}
      }
    }        
  }
  close(server_socket);
  writeLogMessage("HALTED!!!!!");
  return 0;
}
