<?php
/*
 * Project: ClusterControl
 * Version: 2.0
 * Author : Gernot Stocker, gernot.stocker@tugraz.at (c) 2006
 * Date   :
 * Description
 */

class cluster_node_com
{
  var $debug_;
  var $connection_;
  /*
     Constructor
  */
  function cluster_node_com($ip, $port)
  {
    $this->debug_= new debug_class( "cluster_node_com" );
    $this->debug_->printDebug ( 6, "Constructor with $ip $port" );
    $this->connection_=new node_com($ip, $port); 
    
  }

  function isConnected()
  {
    return $this->connection_->isConnected();
  }
  
  function getStatus($class)
  {
    if ($class) 
    { 
      $this->debug_->printDebug ( 6, "Get[ $class ]Status" );
  
      if ( ($this->connection_->connect()) == false ) return null;
            
      $result=$this->connection_->executeCommand( $class );
      $this->connection_->disconnect();
      return($result);
    }
    else
      $this->debug_->printError ( "Invalid StatusClass: $class" );
  }

  function getCPUStatus()
  {
    return $this->explodeCPU( $this->getStatus("cpu") );
  }
  
  function getMemStatus()
  {
    return $this->explodeMem( $this->getStatus("mem") );  
  }
  
  function getLoadStatus()
  {
    return $this->explodeLoad( $this->getStatus("load") );
  }
  
  function getDiskStatus()
  {
    return $this->explodeDisk( $this->getStatus("disk") );
  }

  function getUptimeStatus()
  {
    return $this->explodeUptime( $this->getStatus("uptime"));  
  }  

  function getSystemStatus()
  {
    $result=array();
    $class_result=array();
    $this->debug_->printDebug ( 6, "GetSystem Status" );
    $result["ip"]=$this->connection_->getNodeIp();

    if ($this->connection_->connect()==false) return(null);    
    $result["cpu"]    = $this->explodeCPU($this->connection_->executeCommand("cpu"));
    $result["mem"]    = $this->explodeMem($this->connection_->executeCommand("mem"));
    $result["load"]   = $this->explodeLoad($this->connection_->executeCommand("load"));
    $result["uptime"] = $this->explodeUptime($this->connection_->executeCommand("uptime"));
    $result["disk"]   = $this->explodeDisk($this->connection_->executeCommand("disk" ));
    $this->connection_->disconnect();
    return($result);
  }

  function explodeCPU($input)
  {
    $cpus=array();
    $count=0;
    if ($input == null) return(null);
    
    $input[0]=ereg_replace("  ", " ", $input[0]);
    $entries=explode(" ", $input[0]);
    foreach ($entries as $entry)
    { 
      if (strcmp($entry,"")!=0)
      {
        $cpus["cpu$count"]=trim($entry);
        $count++;
      }
    } 
    return($cpus);
  }
  
  function extractValue($entry)
  {
     $entry_array=split(":",$entry);
     return (trim($entry_array[1]));
  }    

  function explodeMem($input)
  {
    $mem=array();
    $count=0;
    if ($input == null) return(null);
    $memparts=explode(" kB::", $input[0]);
    foreach ($memparts as $entry)
    { 
      if (strcmp($entry,"")!=0)
      {   
        switch ($count) {
          case 0: $mem["total"]=$this->extractValue($entry);
                  break;
          case 1: $mem["free"]=$this->extractValue($entry);
                  break;
          case 2: $mem["buffers"]=$this->extractValue($entry);
                  break;
          case 3: $mem["cached"]=$this->extractValue($entry);
                  break;
        }
        $count++;        
      }
    }
    $total=(float)$mem["total"];
    $free=(float)$mem["free"];
    $cached=(float)$mem["cached"];
    $mem["percentage"]=(($total - $free - $cached)/($total/100));
    return($mem);
  }
      
  function explodeLoad($input)
  {
    $load=array();
    if ($input == null) return(null);
    $entries=explode(" ", $input[0]);
    $load["1min"]=trim($entries[0]);
    $load["5min"]=trim($entries[1]);
    $load["15min"]=trim($entries[2]);
    return($load);
  }
  
  function explodeUptime($input)
  {
    $uptime=array();
    if ($input == null) return(null);
    $entries=explode(" ", $input[0]);
    foreach ($entries as $entry)
    { 
      if ($entry)
       {
         $entry=sprintf("%.0f",$entry);	 
         $modulus = $entry % 86400;
         $uptime["days"]=sprintf("%.0f", (($entry - $modulus) / 86400));
         $entry   = $modulus;
	 $modulus = $entry % 3600;
	 $uptime["hours"]=sprintf("%.0f", (($entry - $modulus) / 3600));
         $entry   = $modulus;
	 $modulus = $entry % 60;
	 $uptime["minutes"]=sprintf("%.0f", (($entry - $modulus) / 60));
	 $uptime["seconds"]=$modulus;
	 break;
       }
    }
    return($uptime);  
  } 
  
  function explodeDisk($input)
  {
    $disk_systems=array();
    $disk=array();
    $count=0;
    if ($input == null) return(null);

    foreach ($input as $entry)
    {
      sscanf($entry, "%s %s %s %s %s %s", $device, $size, $occupied, $free, $perc, $mountpoint);
      $disk["device"]=trim($device);
      $disk["size"]=trim($size);
      $disk["occupied"]=trim($occupied);
      $disk["free"]=trim($free);
      $disk["perc"]=trim($perc);
      $disk["mountpoint"]=trim($mountpoint);
      array_push($disk_systems, $disk);
    }
    return $disk_systems;
  }
}
?>
