<?php
/*
 * Project: ClusterControl
 * Version: 2.0
 * Author : Gernot Stocker, gernot.stocker@tugraz.at (c) 2006
 * Date   :
 * Description
 */

class basic_job_class
{

  var $jobs_dir_;


  var $JOB_=array();
  var $USER_NAME_;
  var $COMMAND_;
  var $POST_PROCESSING_COMMAND_;

  var $queueing_system_;
  var $ping_command_;
  var $qsub_command_;	      
  var $qstat_command_;
  var $nodes_command_;
  var $mpirun_command_;
  var $input_template_;
  var $shell_settings_;

  /*
   * General functions for creating a job to run on PBS/SGE queueingsystem
   */  
  function basic_job_class( $user )
  {
     global $config;    
     $this->basic_constructor_($user, $config['queueing_system']);
  }
  
  function basic_constructor_($user, $queueing_system)
  {

    global $config;    
    $this->queueing_system_=$queueing_system;
    
    $this->USER_NAME_=$user;
    $this->COMMAND_="";
    $this->ping_command=$config['ping_command_'];
      
    #
    # QUEUE Settings
    #
    $this->qsub_command_=$config[$queueing_system . '_qsub_command_'];
    $this->qstat_command_=$config[$queueing_system . '_qstat_command_'];
    $this->nodes_command_=$config[$queueing_system . '_nodes_command_'];
    $this->mpirun_command_=$config[$queueing_system . '_mpirun_command_'];
    $this->jobs_dir_=$config[$queueing_system . '_jobs_dir_'];  
    $this->input_template_=$config[$queueing_system . '_input_template_'];
    $this->shell_settings_=$config[$queueing_system . '_shell_settings_'];
  }
  
  function print_path()
  {
    echo "<br>
          ".$this->ping_command_."            <br>
          ".$this->qsub_command_."        <br>
	  ".$this->qstat_command_."       <br>
	  ".$this->nodes_command_."    <br>
	  " . $this->jobs_dir_ . "            <br>";
  }
  
  function createNewJob($description)
  {
    $new_job=array();
    $directory_path=tempnam($this->jobs_dir_ , $this->USER_NAME_."_");
    unlink($directory_path);
    $old=umask(0);
    if ( mkdir($directory_path, 0770) )
    {
      umask($old);
      $new_job["owner"] = $this->USER_NAME_;
      $new_job["description"] = "$description";
      $new_job["directory_path"] = "$directory_path";
      $new_job["start_file"]  = "$directory_path/runjob";
      $new_job["input_file"]  = "$directory_path/input";
      $new_job["output_file"] = "$directory_path/output";
      $new_job["output_image"] = "$directory_path/output.png";
      $new_job["queue_output_file"] = "$directory_path/queue_output";
      $new_job["queue_nodes_file"] = "$directory_path/queue_nodes";
      $new_job["error_file"] = "$directory_path/error";
      $new_job["done_file"] = "$directory_path/done";
      $new_job["description_file"] = "$directory_path/description";
      $new_job["random"] = substr($directory_path, -6);

      $this->writeDescriptionFiles($new_job);      
      $this->JOB_=$new_job;
    }
    return $new_job;
  }
  
  function writeDescriptionFiles($new_job)
  {
    $description_file=fopen($new_job["description_file"], "w");
    if ($description_file)
    {
      foreach(array_keys($new_job) as $line)
      {
        fwrite($description_file, "$line:\t$new_job[$line]\n" );
      }
    }
  }

  function readJobDescription( $job_directory )
  {
    $description=array();
    if ( $desc_file=@file("$job_directory"."/description") )
    {
      foreach( $desc_file as $line )
      {
        $line=trim($line);
        if ($line)
        {
          $line_array=split(":",$line);
          $key=trim($line_array[0]);
          $description[$key]=trim($line_array[1]);
        }
      }
    }
    return $description;
  }

  //TODO: check the usage of this function   
  function finalizeInputFile ($new_job)
  {
    /*
       fwrite($input_file,"\necho done > ".$new_job["directory_path"]."/done");
       fwrite($input_file,"\ntouch ".$new_job["output_file"]);
       fwrite($input_file,"\ntouch ".$new_job["error_file"]);
     */
  }

  /*
   * Specificfunctions for MPI Jobs
   */
  function writeMpiInputFile( $new_job, $command, $node_amount, $pbs_queue)
  {
     echo "Must be overwritten by the extending class";  
  }

  /*
   * Specificfunctions for PVM Jobs
   */
  function writePVMInputFile( $new_job, $command, $node_amount, $pbs_queue)
  {
     echo "Must be overwritten by the extending class";  
  }
  
  /*
   * Specificfunctions for normal Jobs
   */
  function writeInputFile( $new_job, $command, $node_amount, $queue)
  {
    $input_template=file($this->input_template_);
    $description="m".$new_job["random"].$new_job["description"];
    $input_file=fopen( $new_job["start_file"], "w");
    if ($input_file)
    {
      foreach($input_template as $line)
      {
        $line=str_replace("##-USERNAME-##",      $this->USER_NAME_, $line);
        $line=str_replace("##-WORKDIRECTORY-##", $new_job["directory_path"], $line);
        $line=str_replace("##-DESCRIPTION-##",   str_replace(" ", "_",(substr($description, 0, 15))), $line);
        $line=str_replace("##-NODES-##",         $node_amount, $line);
        $line=str_replace("##-QUEUENAME-##",     $queue, $line);
        $line=str_replace("##-LOGOUTPUT-##",     $new_job["output_file"], $line);
        $line=str_replace("##-STD_OUTPUTFILE-##",   $new_job["output_file"], $line);
        $line=str_replace("##-ERROROUTPUT-##",   $new_job["error_file"], $line);
        $line=str_replace("##-STD_ERRORFILE-##",   $new_job["error_file"], $line);
        $line=str_replace("##-DONEFILE-##",      $new_job["done_file"], $line);
        $line=str_replace("##-COMMAND-##",       $command, $line);
        fwrite($input_file, "$line" );

        if( $this->POSTPROCESSING_COMMAND_ )
        {
           $line=str_replace("##-POSTPROCESSINGCOMMAND-##", $this->POSTPROCESSING_COMMAND_, $line);
        }

      }
      // fwrite($input_file,"$command > ".$new_job["output_file"]." 2>&1\n");
      fwrite($input_file,"\ntouch ".$new_job["output_file"]);
      fwrite($input_file,"\ntouch ".$new_job["error_file"]."\n");
    }
	  
  }



  function createNewMPIJob( $description , $command, $node_amount, $pbs_queue )
  {
    if ($this->createNewJob($description)==0)
    {
      $this->writeMpiInputFile( $this->JOB_, $command, $node_amount, $pbs_queue );
    }
    return $this->JOB_;
  }
  
  function createNewPVMJob( $description, $command, $node_amount, $pbs_queue )
  {
    if ($this->createNewJob($description)==0)
    {
      $this->writePVMInputFile( $this->JOB_, $command, $node_amount, $pbs_queue );  
    }
    return $this->JOB_;
  }


  function postJob()
  {
    $output=array();
    $return_val=0;
    $start_cmd=$this->JOB_["start_file"];
    $output=shell_exec($this->qsub_command_ . " $start_cmd");
    echo "$output<br>";
  }
  
  function deleteJob( $job )
  {
    
    $job_directory=$job["directory_path"];
    
    if ( is_dir($job_directory) )
    { 
      print "<table width=\"580\" border=\"0\" cellspacing=\"1\" class=\"StandardTable\" border=1>";
      print "<tr>";
      print "<td class=\"StandardTableHead\">";
      echo "Job Deletetion";
      print "</td></tr>";
      print "<tr>";
      print "<td class=\"StandardTableCellOdd\">";
      if ( file_exists($job["output_file"]) && file_exists($job["error_file"]) )
      {
        recursiveRMDir($job_directory);
        echo "Deleted: $job_directory";
      }
      else
      {
        echo "Deleting of: ".$job['description']." not possible because its still running!";	
      }
      print "</td></tr></table>";
    }
    return 0;
  }
  
  function set_postprocessing_command( $post_processing_command)
  {
     $this->POST_PROCESSING_COMMAND_=$post_processing_command;
  }

}

?>
