<?php
/*
 * Project: ClusterControl
 * Version: 2.0
 * Author : Gernot Stocker, gernot.stocker@tugraz.at (c) 2006
 * Date   :
 * Description
 */

class pbs_job_class
{

  /*
   * General functions for creating a job to run on PBS
   */  
  function pbs_job_class( $user )
  {
    $this->basic_constructor_($user,'pbs');
  }
    
  /*
   * Specificfunctions for MPI and PVM Jobs
   */
  function writeMpiInputFile( $new_job, $command, $node_amount, $pbs_queue)
  {
    $input_template=file("$this->pbs_input_template_");
    $description="m".$new_job["random"].$new_job["description"];
    $input_file=fopen( $new_job["start_file"], "w");
    if ($input_file)
    {
      foreach($input_template as $line)
      {
	$line=str_replace("##-USERNAME-##",      $this->USER_NAME_, $line);
	$line=str_replace("##-WORKDIRECTORY-##", $new_job["directory_path"], $line);
	$line=str_replace("##-DESCRIPTION-##",   str_replace(" ", "_",(substr($description, 0, 15))), $line);
	$line=str_replace("##-NODES-##",         $node_amount, $line);
	$line=str_replace("##-QUEUENAME-##",     $pbs_queue, $line);
	$line=str_replace("##-LOGOUTPUT-##",     $new_job["pbs_output_file"], $line);
	$line=str_replace("##-ERROROUTPUT-##",   $new_job["error_file"], $line);
	fwrite($input_file, "$line" ); 
      }
      fwrite($input_file, "echo \"PBSNODEFILE:\"\$PBS_NODEFILE; cat \$PBS_NODEFILE\n");
      fwrite($input_file, "cat \$PBS_NODEFILE > ".$new_job["pbs_nodes_file"]."\n");
      // echo("$this->pbs_mpirun_command_ -v -machinefile \$PBS_NODEFILE -np ".$node_amount." $command > ".$new_job["output_file"]." 2>&1\n");
      fwrite($input_file,"$this->pbs_mpirun_command_ -v -machinefile \$PBS_NODEFILE -np ".$node_amount." $command > ".$new_job["output_file"]." 2>&1\n");

      if($this->POST_PROCESSING_COMMAND_)
      {
        fwrite($input_file,$this->POST_PROCESSING_COMMAND_."\n");
      }
      fwrite($input_file,"\necho done > ".$new_job["directory_path"]."/done");
      fwrite($input_file,"\ntouch ".$new_job["output_file"]);
      fwrite($input_file,"\ntouch ".$new_job["error_file"]);
    }
  }


  function writePVMInputFile( $new_job, $command, $node_amount, $pbs_queue)
  {
    $input_template=file("$this->pbs_input_template_");
    $description="p".$new_job["random"].$new_job["description"];
    $input_file=fopen( $new_job["start_file"], "w");
    if ($input_file)
    {
      foreach($input_template as $line)
      {
	$line=str_replace("##-USERNAME-##",      $this->USER_NAME_, $line);
	$line=str_replace("##-WORKDIRECTORY-##", $new_job["directory_path"], $line);
	$line=str_replace("##-DESCRIPTION-##",   str_replace(" ", "_",(substr($description, 0, 15))), $line);
	$line=str_replace("##-NODES-##",         $node_amount, $line);
	$line=str_replace("##-QUEUENAME-##",     $pbs_queue, $line);	
	$line=str_replace("##-LOGOUTPUT-##",     $new_job["pbs_output_file"], $line);
	$line=str_replace("##-ERROROUTPUT-##",   $new_job["error_file"], $line);
	fwrite($input_file, "$line" ); 
      }
      fwrite($input_file, "cat \$PBS_NODEFILE > ".$new_job["pbs_nodes_file"]."\n");        
      fwrite($input_file,"source $this->pbs_shell_settings_\n");
        
	/* PVM INIT 
      
      fwrite($input_file,'if [ ! -x \"/tmp/pvmd.503\" ]; then'."\n");
      fwrite($input_file,'  ls -1 /master/nodeconfig/running-nodes | cut -d "-" -f2 >> $PBS_NODEFILE'."\n");
      fwrite($input_file,"fi\n");
      */
      
      fwrite($input_file,'  PBS_RUN_NODEFILE="$PVM_TMP/nodes$$"'."\n");
      fwrite($input_file,'  echo "* ep=/master/software/bioinfo/bin wd=/netapp/BioInfo/data/genbank sp=2000" > $PBS_RUN_NODEFILE'."\n");
      fwrite($input_file,'  cat $PBS_NODEFILE >> $PBS_RUN_NODEFILE'."\n");      
      fwrite($input_file,'  cat $PBS_RUN_NODEFILE'."\n");      
      fwrite($input_file,'  echo quit | pvm $PBS_RUN_NODEFILE'."\n");        	
      fwrite($input_file,"$command > ".$new_job["output_file"]." 2>&1\n");
      fwrite($input_file,'echo halt | pvm'."\n");        	
      fwrite($input_file,'rm $PBS_RUN_NODEFILE '."\n");        	
    
        if($this->POST_PROCESSING_COMMAND_)
        {
          fwrite($input_file,$this->POST_PROCESSING_COMMAND_."\n");
        }
        fwrite($input_file,"\necho done > ".$new_job["directory_path"]."/done");
        fwrite($input_file,"\ntouch ".$new_job["output_file"]);
        fwrite($input_file,"\ntouch ".$new_job["error_file"]);
    }
  }
  

  function createNewMPIJob( $description , $command, $node_amount, $pbs_queue )
  {
    if ($this->createNewJob($description)==0)
    {
      $this->writeMpiInputFile( $this->JOB_, $command, $node_amount, $pbs_queue );
    }
    return $this->JOB_;
  }
  


  function createNewPVMJob( $description, $command, $node_amount, $pbs_queue )
  {
    if ($this->createNewJob($description)==0)
    {
      $this->writePVMInputFile( $this->JOB_, $command, $node_amount, $pbs_queue );  
    }
    return $this->JOB_;
  }

  /*
   * Specificfunctions for normal Jobs
   */
  function writeInputFile( $new_job, $command, $node_amount, $pbs_queue)
  {
    $input_template=file("$this->pbs_input_template_");
    $description="m".$new_job["random"].$new_job["description"];
    $input_file=fopen( $new_job["start_file"], "w");
    if ($input_file)
    {
      foreach($input_template as $line)
      {
	$line=str_replace("##-USERNAME-##",      $this->USER_NAME_, $line);
	$line=str_replace("##-WORKDIRECTORY-##", $new_job["directory_path"], $line);
	$line=str_replace("##-DESCRIPTION-##",   str_replace(" ", "_",(substr($description, 0, 15))), $line);
	$line=str_replace("##-NODES-##",         $node_amount, $line);
	$line=str_replace("##-QUEUENAME-##",     $pbs_queue, $line);
	$line=str_replace("##-LOGOUTPUT-##",     $new_job["pbs_output_file"], $line);
	$line=str_replace("##-ERROROUTPUT-##",   $new_job["error_file"], $line);
	fwrite($input_file, "$line" ); 
      }
      
      fwrite($input_file, "cat \$PBS_NODEFILE > ".$new_job["pbs_nodes_file"]."\n");
      fwrite($input_file,"$command > ".$new_job["output_file"]." 2>&1\n");
  
      if($this->POST_PROCESSING_COMMAND_)
      {
        fwrite($input_file,$this->POST_PROCESSING_COMMAND_."\n");
      }
      fwrite($input_file,"echo done > ".$new_job["directory_path"]."/done");
      fwrite($input_file,"\ntouch ".$new_job["output_file"]);
      fwrite($input_file,"\ntouch ".$new_job["error_file"]);
    }
  }



  function postJob()
  {
    $output=array();
    $return_val=0;
    $start_cmd=$this->JOB_["start_file"];
    exec("$this->pbs_qsub_command_ $start_cmd", $output, $return_val);
    foreach ($output as $line)
    {
     echo "$line<br>";
    }
  }
    
}

?>
