<?php
/*
 * Project: ClusterControl
 * Version: 2.0
 * Author : Gernot Stocker, gernot.stocker@tugraz.at (c) 2006
 *          Dietmar Rieder, dietmar.rieder@tugraz.at
 * Date   :
 *
 * Description:
 *     This class can be used as interface API to the clustercontrol backend
 *     in order to define, submit and view the results within the framework
 *
 */
class run_module_class {
   var $PROGRAMS_;
   var $DATABASES_;
   var $JOB_CLASS_;
   var $NEW_JOB_;
   var $JOB_DIRECTORY_="";
   var $ACTUAL_PROGRAM_="";
   var $ACTUAL_DATABASE_="";
   var $ACTUAL_JOBNAME_="";
   var $ACTUAL_COMMAND_LINE_="";
   var $ACTUAL_POSTPROCESSING_COMMAND_="";
   var $DEFAULT_REFRESH_TIME_=2;
   var $QUEUE_;
   var $NODES_AMOUNT_;
   var $DEFAULT_URL_="";
   
   
   
   /*
    * run_module_class(USERNAME, PROGRAMNAME, JOBNAME)
    *
    * Class Constructor, which needs the name of user, the name of the 
    * program and the name of the job that should be submitted through 
    * this Class.
    *
    *   By default the additional data for running the application PROGRAMNAME
    *   is fetched from the conf/programs.conf file and with the PROGRAMNAME
    *   parameter you set the internal settings to the values acording the 
    *   $program line in the conf/programs.conf
    *
    *   If this file doesn't exist the program settings must be set with the
    *   define_current_program function (see below) in order to submit the job.
    *   The default behaviour without calling the define_current_program is to 
    *   initialize the full_programmpath with the content of the $program variable
    *   and the default_parameters are left free.
    *
    *   For database definition conf/databases.conf is read in. See more details in 
    *   the Moduledevelopment-HOWTO
    *
    */
  function run_module_class($user, $program, $jobname, $queueingsystem)
  {
	global $config;

	if($jobname == '') {
	    $jobname = date('Ymd-Hms');
	}

	$this->ACTUAL_PROGRAM_=$program;
	$this->ACTUAL_JOBNAME_=$jobname;
        $this->DEFAULT_URL_=$config['DEFAULT_URL'];

	switch ($queueingsystem)
        {
	  case "local":
	     $this->QUEUE_="none";
 	     $this->JOB_CLASS_=new local_job_class($user);
             break;	  
          case "pbs":
	     $this->QUEUE_=$config[$config['queueing_system'].'_default_queue_'];
 	     $this->JOB_CLASS_=new pbs_job_class($user);
	     $this->NODES_AMOUNT_=$config[$config['queueing_system'].'_default_nodes_amount_'];
             break;
          case "sge":
          default:
	     $this->QUEUE_="none";
 	     $this->JOB_CLASS_=new sge_job_class($user);
	     $this->NODES_AMOUNT_=$config[$config['queueing_system'].'_default_nodes_amount_'];	     
             break;  	
	}
		
	$this->DEFAULT_REFRESH_TIME_=$config['DEFAULT_REFRESH_TIME'];
        $this->NEW_JOB_=$this->JOB_CLASS_->createNewJob($program."-".$jobname);
  	$this->JOB_DIRECTORY_=urlencode($this->NEW_JOB_["directory_path"]);

	$this->PROGRAMS_=new programs_class();	
	if (! file_exists("conf/programs.conf"))
	{
	  $this->PROGRAMS_->InternalAddNewProgram($this->ACTUAL_PROGRAM_);
	}
	$this->set_command_line();        
	if (file_exists("conf/databases.conf"))
	{
	   $this->DATABASES_=new databases_class();		
	}	
   }

   /*
    * print_html_header()
    *
    * The funcion print_html_header creates the HTML header which redirects
    * your browser directly to the result page after having committed the job on
    * the cluster. Therefore it uses the refreshtime specified in the 
    * globally defined $conig->DEFAULT_REFRESH_TIME_
    *
    */
   function print_html_header()
   {
     $dir=urlencode($this->NEW_JOB_["directory_path"]);
     echo "
          <html>
            <head>
              <title>RUN Blast Search</title>";
     echo "
              <META http-equiv=\"refresh\" content=\"".$this->DEFAULT_REFRESH_TIME_.";url=".$this->DEFAULT_URL_."/job_actions.php?action=details&dir=" . $dir . "\">";
     echo '
                <meta http-equiv="Content-Style-Type" content="text/css">
                <link REL="STYLESHEET" HREF="../../Genome.css" type="text/css">
            </head>
          <body bgcolor=white>
	  <table width="580" border="0" cellspacing="1" class="StandardTable" border=1>
           <tr>
            <td class="StandardTableHead">Job submission</td>
	   </tr>
	   <tr><td class="StandardTableCellOdd">
	   <pre>
          ';
   }
   
    
   /*
    * print_html_footer()
    *
    * The funcion print_html_footer  completes the created the HTML result page
    *
    */
   function print_html_footer()
   {
     echo "
          </pre>
	  </td></tr>
          </table>
          </body>
          <html>
	  ";
   }
   

   /*
    * print_job_description()
    *
    * The funcion print_job_description prints out some basice information about the job 
    * configuration that is going to be submitted.
    *
    */
   function print_job_description()
   {
     $pro=$this->PROGRAMS_->getProgramData($this->ACTUAL_PROGRAM_);
     echo " RUN [ ".$this->NEW_JOB_["description"]." ]
	       <table>
		  <tr><td>Programm:</td><td>".$pro['full_programmpath']."(".$pro['default_arguments'].")</td></tr>
		  <tr><td>DB</td><td>".$this->ACTUAL_DATABASE_."</td><tr>
		  <tr><td>CMD</td><td>".$this->ACTUAL_COMMAND_LINE_."</td><tr>
	       </table>";
   }
   
   
   /*
    * define_current_program($full_path_to_program, $default_arguments)
    *
    * With this function a programmer can define the job command if he has no
    * conf/programs.conf file in the module directory.
    * It gets the full path to the program run while the job is executed 
    * and the default arguments, added always to the program
    *
    */

   function define_current_program($full_path_to_program, $default_arguments)
   {
     $this->add_new_program($this->ACTUAL_PROGRAM_, $full_path_to_program, $default_arguments);
     $this->set_command_line();
   }


   /*
    * process_databases( $searchdatabases )
    *
    * looks in the conf/databases.conf file for the database with the name
    * $searchdatabases and sets internally the Database variable to the path
    * which is specified in the config file.
    *
    * This value is used for replacing the %Database string in the add_parameter_to_command
    * function
    *
    */
   function process_databases( $searchdatabases )
   {
     $exec_dbs="";
     if ( $searchdatabases && is_array($searchdatabases))
     {
       foreach( array_keys( $searchdatabases ) as $searchdatabases_index )
       {
         $actual_dbname=$searchdatabases[$searchdatabases_index];
         $actual_dbs= $this->DATABASES_->getDatabaseData("$actual_dbname");
 
         foreach( array_keys( $actual_dbs ) as $actual_dbs_index )
         {
           $actual_db=$actual_dbs[$actual_dbs_index];
           $actual_dbpath=$actual_db['database_path'];
           $actual_dbfile=$actual_db['database_file'];
           $exec_dbs=$exec_dbs." ".$actual_dbpath."/".$actual_dbfile." ";
         }
      }
     }
     trim($exec_dbs);
     $this->ACTUAL_DATABASE_=$exec_dbs;
   }
   
   
   /*
    * process_sequence_input( $sequence, $sequence_file )
    *
    * is used to process fileupload dialog combined with a multitext field. It 
    * takes the content of the $sequence variable or the content of the temporary 
    * file at the $sequence_file path and creates an input file for the job.
    *
    * The name of the internal inputfile is used for replacing the %Inputfile 
    * string in the add_parameter_to_command function
    *
    */
   function process_sequence_input( $sequence_name, $post_array, $sequence_file_name, $files_array)
   {
     if ( array_key_exists($sequence_name,$post_array) )
     {
        $sequence=$post_array[$sequence_name];
     }
     else
     {
        unset($sequence);
     };
     
     if (array_key_exists($sequence_file_name,$files_array))
     {
        $sequence_file=$files_array[$sequence_file_name]['tmp_name'];
	$sequence_file_size=$files_array[$sequence_file_name]['size'];
     }
     else
     {
        $sequence_file=NULL;
	$sequence_file_size=1000000;
     };
     
     if (isset($sequence) && ( strcmp( trim($sequence) ,"" )!=0 ))
     {
       $form_input_file=fopen($this->NEW_JOB_["input_file"],"w");
       fwrite ($form_input_file, $sequence);
       fclose($form_input_file);
     }   
     
     if (($sequence_file!=NULL) && (strcmp($sequence_file,"none")!=0) && (strcmp($sequence_file,"")!=0) && ($sequence_file_size < 100001))
     {
        if (@copy ($sequence_file, $this->NEW_JOB_["input_file"]))
        {
          # Delete the temporary file
          unlink ($sequence_file);
        }
	else
        {
          unlink ($sequence_file);
        }
     }
   }
   
   /*
    * add_parameter_to_command( $parameter, $value )
    *
    * This function is used to extend the command line used for job submission
    * with a parameter switch and optionally a parametervalue
    * e.g.
    *      ls -l     ... would go with add_parameter_to_command( "-l", "" )
    *      cut -d 2  ... would go with add_parameter_to_command( "-d", "2" );
    *
    * This function replaces also the strings %Inputfile, %Outputfile and %Database
    * in the value by the earlier defined job internal values
    * see run_module_class(), process_databases(), process_sequence_input()
    * 
    */
   function add_parameter_to_command( $parameter, $value )
   {
     $parameter=trim($parameter);
     $value=trim($value);
     $value=str_replace("'", "",$value);
     switch ($value) 
     {
       case "%Inputfile":
          $value="'".trim($this->NEW_JOB_["input_file"])."'";
          break;
       case "%Outputfile":
          $value="'".trim($this->NEW_JOB_["output_file"])."'";
          break;
       case "%Outputimage":
          $value="'".trim($this->NEW_JOB_["output_image"])."'";
          break;
        case "%Database":
          $value="'".trim($this->ACTUAL_DATABASE_)."'";
          break;
       case "":
          $value="";
          break;
       default:
          $value="'".$value."'";
     }
     $this->ACTUAL_COMMAND_LINE_=$this->ACTUAL_COMMAND_LINE_." $parameter ".$value;
   }
   
   /*
    * set_postprocessing_command( $command )
    *
    * if you also want to execute after execution of the normal program an additional 
    * postprocessing program, set the full path with this method to the program that 
    * should be run
    *
    */
   function set_postprocessing_command( $command )
   {
     $this->ACTUAL_POSTPROCESSING_COMMAND_=$command;
   }

   /*
    * add_parameter_to_postprocessing_command( $parameter, $value )
    *
    * This function behaves in the same way as the add_parameter_to_command functions
    * but it extends not the ACTUAL_COMMAND_LINE_ but the ACTUAL_POSTPROCESSING_COMMAND_ 
    *
    */
   function add_parameter_to_postprocessing_command( $parameter, $value )
   {
     $parameter=trim($parameter);
     $value=trim($value);
     switch ($value) 
     {
       case "%Inputfile":
          $value="'".trim($this->NEW_JOB_["input_file"])."'";
          break;
       case "%Outputfile":
          $value="'".trim($this->NEW_JOB_["output_file"])."'";
          break;
       case "%Database":
          $value="'".trim($this->ACTUAL_DATABASE_)."'";
          break;
       case "":
          $value="";
          break;
       default:
          $value="'".$value."'";
     }
     $this->ACTUAL_POSTPROCESSING_COMMAND_=$this->ACTUAL_POSTPROCESSING_COMMAND_." $parameter $value";
   }
   
   /*
    * commit_job( $type_of_job )
    *
    * This function submits the job to the clustercontrol framework with the default parameters
    * set in $config['NODES_AMOUNT_'] and $config['QUEUE_']. The type_of_job parameter defines whether
    * clustercontrol has to add an additional environment for parallel applications. 
    *
    * Supported parallel environments are:
    *
    *  MPI ... MPICH internal commands are used for submitting and running the job on only the 
    *          nodes automatically assigned by the PBS.
    *
    *  PVM ... A Parallel Virtual machine will be started only at the nodes that the job gets assigned 
    *          through PBS and starts than the pvm-application defined in the job by the Constructor, 
    *          if a conf/programs.conf file is present or by the add_new_program and define_current_program
    *          functions
    */

   function commit_job( $type_of_job )
   { 
      $this->advanced_commit_job($type_of_job, $this->NODES_AMOUNT_, $this->QUEUE_);
   }
   
   
   /*
    * advanced_commit_job( $type_of_job )
    *
    * is doing exactly the same as commit_job is doing, but the programmer can specify the 
    * amount of nodes $nodes that should process the job and the queuename $queue to which the job 
    * should be submitted
    *
    */
   function advanced_commit_job($type_of_job, $nodes, $queue)
   {
     if ($this->ACTUAL_POSTPROCESSING_COMMAND_!="")
     {
       $this->JOB_CLASS_->set_postprocessing_command($this->ACTUAL_POSTPROCESSING_COMMAND_);
     } 
     switch ($type_of_job) 
     {
       case "MPI":
          $this->JOB_CLASS_->writeMpiInputFile( $this->NEW_JOB_, $this->ACTUAL_COMMAND_LINE_, $nodes, $queue);
          break;
       case "PVM":
          $this->JOB_CLASS_->writePVMInputFile( $this->NEW_JOB_, $this->ACTUAL_COMMAND_LINE_, $nodes, $queue);
          break;
       default:
          $this->JOB_CLASS_->writeInputFile( $this->NEW_JOB_, $this->ACTUAL_COMMAND_LINE_, 1, $this->QUEUE_);
          break;
     }
     $this->JOB_CLASS_->postJob(); 
   }
   
   /*
    * set_default_queue( $newqueue )
    * set_default_nodes_amount($newamount)
    *
    * The following two functions are used to set the QUEUEING System parameters alos after
    * definition of the job. In this way you can set the JOB Parameters without using 
    * the advances_commit_job
    */
   function set_default_queue( $newqueue )
   {
     $this->QUEUE_=$newqueue;
   }

   function set_default_nodes_amount( $newamount )
   {
     $this->NODES_AMOUNT_=$newamount;
   }
   
   
   
   
   /*
    *  -------------------------------------------------------------------------
    *  INTERNAL FUNCTIONS don't use them
    *  -------------------------------------------------------------------------
    */
      
   
   /*
    * get_job_directoryname()
    *
    * Basically all that clustercontrol needs for identifying, processing etc. a job 
    * is the directory name of the job. With this directoyname clustercontrol can 
    * access its description files and all teh information it needs for visualizing a job
    * 
    * This function gives this directory name back to the prorgammer
    * 
    * In general this function is only for internal use in the class
    *
    */
   function get_job_directoryname()
   {
     return $this->$JOB_DIRECTORY_;
   }
      
   function add_new_program($new_programname, $full_path_to_program, $default_arguments)
   {
     $this->PROGRAMS_->addNewProgram($new_programname, $full_path_to_program, $default_arguments);
   }
   
   function set_command_line()
   {
     $program_data=$this->PROGRAMS_->getProgramData($this->ACTUAL_PROGRAM_);
     $this->ACTUAL_COMMAND_LINE_=$program_data["full_programmpath"]." ".$program_data["default_arguments"];
   }
   
}

?>
